local _baseControl
local _alphaAnimation
local _isActive = false
local _temp = {}

function LovelyEmotes.EmoteSelectionWindow.UpdateMaxAlpha()
	local maxAlpha = LovelyEmotes.SavedAccountVariables.Alpha / 100

	_alphaAnimation:SetMinMaxAlpha(0, maxAlpha)

	if _isActive == true then
		_baseControl:SetAlpha(maxAlpha)
	end
end

local function SelectEmote(emote, playEmote)
	if not emote then
		_temp.EmoteID = emote
		playEmote = false
	else
		_temp.EmoteID = emote.ID
	end

	local displayName = ""
	local slashName = LE_Const_EmoteUnknownDisplayName

	if emote then
		slashName = emote.SlashName
		if emote.DisplayName ~= emote.SlashName then displayName = emote.DisplayName end
	elseif emote == false then
		slashName = LE_Const_EmoteEmptyDisplayName
	end

	LE_EmoteSelectionWindowTopBoxSelectedEmoteTextDisplayName:SetText(displayName)
	LE_EmoteSelectionWindowTopBoxSelectedEmoteTextSlashName:SetText(slashName)

	if not playEmote then return end
	emote.Play()
end

function LovelyEmotes.EmoteSelectionWindow.ShowWindow(favoriteIndex, emote)
	if _isActive == false then
		_isActive = true

		LovelyEmotes.PlayWindowOpenSound()
		LovelyEmotes.EmoteList:ResetList()

		_alphaAnimation:FadeIn(0,
			DEFAULT_SCENE_TRANSITION_TIME,
			ZO_ALPHA_ANIMATION_OPTION_USE_CURRENT_ALPHA,
			nil,
			ZO_ALPHA_ANIMATION_OPTION_USE_CURRENT_ALPHA)

		_baseControl:SetHidden(false)
	end

	_temp.FavoriteIndex = favoriteIndex
	SelectEmote(emote, false)
end

local function HideWindow(instant)
	if _isActive == false then return end
	_isActive = false

	LovelyEmotes.PlayWindowCloseSound()
	LovelyEmotes.MainWindow.DeselectFavoriteButton()

	ZO_ClearTable(_temp)

	if instant then
		_baseControl:SetAlpha(0)
		_baseControl:SetHidden(true)
	else
		_alphaAnimation:FadeOut(0,
			DEFAULT_SCENE_TRANSITION_TIME,
			ZO_ALPHA_ANIMATION_OPTION_USE_CURRENT_ALPHA,
			function() _baseControl:SetHidden(true) end,
			ZO_ALPHA_ANIMATION_OPTION_USE_CURRENT_ALPHA)
	end
end
LovelyEmotes.EmoteSelectionWindow.HideWindow = HideWindow

function LovelyEmotes.EmoteSelectionWindow.SaveEmote()
	if _isActive == false then return end

	if _temp.EmoteID ~= nil then
		LovelyEmotes.SaveEmote(_temp.FavoriteIndex, _temp.EmoteID)
		LovelyEmotes.MainWindow.RefreshFavoriteButton(_temp.FavoriteIndex)
	end

	HideWindow()
end

function LovelyEmotes.EmoteSelectionWindow.EmptySlot()
	if _isActive == false then return end

	SelectEmote(false, false)
end

local function SetupEmoteList()
	LovelyEmotes.EmoteList:SetParent(_baseControl, 16, function(button, data) SelectEmote(data, button == 1) end)
end
LovelyEmotes.EmoteSelectionWindow.SetupEmoteList = SetupEmoteList

local function SavePosition()
	local savedAccountVariables = LovelyEmotes.SavedAccountVariables
	savedAccountVariables.EmoteListPositionX = _baseControl:GetLeft()
	savedAccountVariables.EmoteListPositionY = _baseControl:GetTop()
end

function LovelyEmotes.EmoteSelectionWindow.Initialize()
	_baseControl = LE_EmoteSelectionWindow
	local savedAccountVariables = LovelyEmotes.SavedAccountVariables

	_baseControl:ClearAnchors()
	_baseControl:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, savedAccountVariables.EmoteListPositionX, savedAccountVariables.EmoteListPositionY)

	_baseControl:SetAlpha(0)
	_alphaAnimation = ZO_AlphaAnimation:New(_baseControl)
	_alphaAnimation:SetMinMaxAlpha(0, savedAccountVariables.Alpha / 100)

	SetupEmoteList()

	_baseControl:SetHandler("OnMoveStop", SavePosition)
end
